"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/main.ts
var main_exports = {};
__export(main_exports, {
  getStore: () => getStore
});
module.exports = __toCommonJS(main_exports);

// src/client.ts
var import_node_buffer = require("buffer");
var import_node_process = require("process");

// src/retry.ts
var DEFAULT_RETRY_DELAY = 5e3;
var MIN_RETRY_DELAY = 1e3;
var MAX_RETRY = 5;
var RATE_LIMIT_HEADER = "X-RateLimit-Reset";
var fetchAndRetry = async (fetch, url, options, attemptsLeft = MAX_RETRY) => {
  try {
    const res = await fetch(url, options);
    if (attemptsLeft > 0 && (res.status === 429 || res.status >= 500)) {
      const delay = getDelay(res.headers.get(RATE_LIMIT_HEADER));
      await sleep(delay);
      return fetchAndRetry(fetch, url, options, attemptsLeft - 1);
    }
    return res;
  } catch (error) {
    if (attemptsLeft === 0) {
      throw error;
    }
    const delay = getDelay();
    await sleep(delay);
    return fetchAndRetry(fetch, url, options, attemptsLeft - 1);
  }
};
var getDelay = (rateLimitReset) => {
  if (!rateLimitReset) {
    return DEFAULT_RETRY_DELAY;
  }
  return Math.max(Number(rateLimitReset) * 1e3 - Date.now(), MIN_RETRY_DELAY);
};
var sleep = (ms) => new Promise((resolve) => {
  setTimeout(resolve, ms);
});

// src/client.ts
var NETLIFY_CONTEXT_VARIABLE = "NETLIFY_BLOBS_CONTEXT";
var Client = class _Client {
  constructor(context, fetch) {
    this.context = context ?? {};
    this.fetch = fetch;
  }
  static getEnvironmentContext() {
    if (!import_node_process.env[NETLIFY_CONTEXT_VARIABLE]) {
      return;
    }
    const data = import_node_buffer.Buffer.from(import_node_process.env[NETLIFY_CONTEXT_VARIABLE], "base64").toString();
    try {
      return JSON.parse(data);
    } catch {
    }
  }
  getContext() {
    const context = {
      ..._Client.getEnvironmentContext(),
      ...this.context
    };
    if (!context.siteID || !context.token) {
      throw new Error(`The blob store is unavailable because it's missing required configuration properties`);
    }
    return context;
  }
  async getFinalRequest(storeName, key, method) {
    const context = this.getContext();
    const encodedKey = encodeURIComponent(key);
    if ("edgeURL" in context) {
      return {
        headers: {
          authorization: `Bearer ${context.token}`
        },
        url: `${context.edgeURL}/${context.siteID}/${storeName}/${encodedKey}`
      };
    }
    const apiURL = `${context.apiURL ?? "https://api.netlify.com"}/api/v1/sites/${context.siteID}/blobs/${encodedKey}?context=${storeName}`;
    const headers = { authorization: `Bearer ${context.token}` };
    const fetch = this.fetch ?? globalThis.fetch;
    const res = await fetch(apiURL, { headers, method });
    if (res.status !== 200) {
      throw new Error(`${method} operation has failed: API returned a ${res.status} response`);
    }
    const { url } = await res.json();
    return {
      url
    };
  }
  async makeRequest({ body, headers: extraHeaders, key, method, storeName }) {
    const { headers: baseHeaders = {}, url } = await this.getFinalRequest(storeName, key, method);
    const headers = {
      ...baseHeaders,
      ...extraHeaders
    };
    if (method === "put" /* PUT */) {
      headers["cache-control"] = "max-age=0, stale-while-revalidate=60";
    }
    const options = {
      body,
      headers,
      method
    };
    if (body instanceof ReadableStream) {
      options.duplex = "half";
    }
    const fetch = this.fetch ?? globalThis.fetch;
    const res = await fetchAndRetry(fetch, url, options);
    if (res.status === 404 && method === "get" /* GET */) {
      return null;
    }
    if (res.status !== 200) {
      throw new Error(`${method} operation has failed: store returned a ${res.status} response`);
    }
    return res;
  }
};

// src/store.ts
var EXPIRY_HEADER = "x-nf-expires-at";
var Store = class _Store {
  constructor(options) {
    this.client = options.client;
    this.name = "deployID" in options ? `deploy:${options.deployID}` : options.name;
  }
  static getExpirationHeaders(expiration) {
    if (typeof expiration === "number") {
      return {
        [EXPIRY_HEADER]: (Date.now() + expiration).toString()
      };
    }
    if (expiration instanceof Date) {
      return {
        [EXPIRY_HEADER]: expiration.getTime().toString()
      };
    }
    if (expiration === void 0) {
      return {};
    }
    throw new TypeError(`'expiration' value must be a number or a Date, ${typeof expiration} found.`);
  }
  async delete(key) {
    await this.client.makeRequest({ key, method: "delete" /* DELETE */, storeName: this.name });
  }
  async get(key, options) {
    const { type } = options ?? {};
    const res = await this.client.makeRequest({ key, method: "get" /* GET */, storeName: this.name });
    const expiration = res?.headers.get(EXPIRY_HEADER);
    if (typeof expiration === "string") {
      const expirationTS = Number.parseInt(expiration);
      if (!Number.isNaN(expirationTS) && expirationTS <= Date.now()) {
        return null;
      }
    }
    if (res === null) {
      return res;
    }
    if (type === void 0 || type === "text") {
      return res.text();
    }
    if (type === "arrayBuffer") {
      return res.arrayBuffer();
    }
    if (type === "blob") {
      return res.blob();
    }
    if (type === "json") {
      return res.json();
    }
    if (type === "stream") {
      return res.body;
    }
    throw new Error(`Invalid 'type' property: ${type}. Expected: arrayBuffer, blob, json, stream, or text.`);
  }
  async set(key, data, { expiration } = {}) {
    const headers = _Store.getExpirationHeaders(expiration);
    await this.client.makeRequest({
      body: data,
      headers,
      key,
      method: "put" /* PUT */,
      storeName: this.name
    });
  }
  async setJSON(key, data, { expiration } = {}) {
    const payload = JSON.stringify(data);
    const headers = {
      ..._Store.getExpirationHeaders(expiration),
      "content-type": "application/json"
    };
    await this.client.makeRequest({
      body: payload,
      headers,
      key,
      method: "put" /* PUT */,
      storeName: this.name
    });
  }
};
var getStore = (input) => {
  if (typeof input === "string") {
    const client = new Client();
    return new Store({ client, name: input });
  }
  if (typeof input.name === "string") {
    const { fetch, name, ...context } = input;
    const client = new Client(context, fetch);
    return new Store({ client, name });
  }
  if (typeof input.deployID === "string") {
    const { fetch, deployID, ...context } = input;
    const client = new Client(context, fetch);
    return new Store({ client, name: deployID });
  }
  throw new Error("`getStore()` requires a `name` or `siteID` properties.");
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getStore
});
